/**********************************************************************
 *HY16F184
 * main.c
 * Created on:2013/12/08
 * Maintenance:2014/10/01
 * --------------------------------------------------------------------
 * Release 1.0.0
 * Program Description:
 * --------------------
  * HY16F184(I2C Slave)     I2C Master
 * ---------
 *          |              --------------------
 *    PT1.3 | INT ---> INT | PT2.4            |
 *    PT2.7 | SDA <--> SDA | PT2.3 HY16F198   |
 *    PT2.6 | SCK <--- SCK | PT2.2            |
 *     GND  |              |                  |
 *          |              |                  |
 * ---------               --------------------
 **********************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY16F188.h"
#include "System.h"
#include "DrvGPIO.h"
#include "DrvI2C.h"
#include "DrvCLOCK.h"
#include "my define.h"
#include "DrvADC.h"
#include "DrvTimer.h"
#include "DrvADC.h"
#include "DrvPMU.h"
/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_I2C_TxDone:1;
    unsigned b_I2C_RxDone:1;
  };
} MCUSTATUS;

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define  Disable 0
#define  Enable  1
#define I2CBufferSize 256

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
MCUSTATUS  MCUSTATUSbits;

// Data range 32767~-32768, +- 15bit format(bit30~bit16), Bit31=+-Sign, Bit0=Flag; 0:old, 1:new
int ADC_CH1, ADC_CH2, ADC_CH3, ADC_CH4;
int ADC_CH1_Array[4], ADC_CH2_Array[4], ADC_CH3_Array[4], ADC_CH4_Array[4];
int ADC_CH1_Count, ADC_CH2_Count, ADC_CH3_Count, ADC_CH4_Count;
char CH1_LowByte, CH2_LowByte, CH3_LowByte, CH4_LowByte;
char CH1_MiddleByte, CH2_MiddleByte, CH3_MiddleByte, CH4_MiddleByte;
char CH1_HighByte, CH2_HighByte, CH3_HighByte, CH4_HighByte;
unsigned char ADC_CH4_Finish;

int i;
unsigned char ADfinish;
int  ADCData;

unsigned char Buffer[I2CBufferSize];
unsigned char I2C_RW;
unsigned char I2C_TARGET;                  // Target I2C slave address
unsigned char I2C_Sendbuf[I2CBufferSize];
unsigned char I2C_Recbuf[I2CBufferSize];
unsigned int DataTxLen,DataTxIndex,DataRxLen,DataRxIndex,EndFlag;
unsigned int GeneralFlag;
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void InitalADC0(void);
void HWI2C_INI(void);
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{

  // GPIO  Trigger/Output Rate setting
  pio_00=0x00000808;  //PT1OE3=1, //PT1.3 Output Mode, check output rate=Interrupt pin
  pio_04=0x00000808; //PT1DO3=1  PT1.3 pull High, check output rate=Interrupt pin


  //ADC initialization
  op_00=0x00000202; // OPOE=1, let OPOI=OPO, setting pin15
  pio_24=0x80800000;  //PT3IE7=1b, let OPOI=OPO, setting pin15
  InitalADC0(); //ADC CH0, 16F188, pin20/pin19

  //Data initialization=0
  ADC_CH1=0, ADC_CH2=0, ADC_CH3=0, ADC_CH4=0, ADCData=0, i=0;
  CH1_LowByte=0, CH2_LowByte=0, CH3_LowByte=0, CH4_LowByte=0;
  CH1_MiddleByte=0, CH2_MiddleByte=0, CH3_MiddleByte=0, CH4_MiddleByte=0;
  CH1_HighByte=0, CH2_HighByte=0, CH3_HighByte=0, CH4_HighByte=0;
  ADC_CH1_Count=0, ADC_CH2_Count=0, ADC_CH3_Count=0, ADC_CH4_Count=0;
  ADC_CH4_Finish=0;  //Disable I2C Slave data Buffer Update
  ADfinish=0;

  for(i=0;i<=256;i++)  //if 6=HY14E10 Test tool AP
  {
    I2C_Sendbuf[i]=0;	//Initial I2C data buffer=0
    I2C_Recbuf[i]=0;
  }

  for(i=0;i<=3;i++)
  {
    ADC_CH1_Array[i]=0;
 	ADC_CH2_Array[i]=0;
 	ADC_CH3_Array[i]=0;
 	ADC_CH4_Array[i]=0;
  }

   DataRxIndex=0;
   DataTxIndex=0;
   EndFlag=0;
   DataTxLen=256;

  // I2C initialization
  HWI2C_INI();

  // SPI Interrupt Enable, Dummy setting, in order to enter wait mode
  int_00=0x03030303;  //STXIE=1, SRXIE=1, STXIF=1, SRXIF=1


  DrvADC_CombFilter(Disable);  //Disable CombFilter
  DrvADC_CombFilter(Enable);  //Enable CombFilte
  SYS_EnableGIE(7,0x3F);  // Enable GIE(Global Interrupt)

      for(i=0;i<=15;i++)
	  {
	    switch (i)
	    {
	      case 0:  //ŪW-1
	      while(!ADfinish);
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH1_Array[0]=ADCData;
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 1:  //ŪW-2
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH1_Array[1]=ADCData;//>>16;
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 2:  //ŪW-3
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH1_Array[2]=ADCData;
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 3:  //ŪW-4
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH1_Array[3]=ADCData;
	      DrvADC_CombFilter(Disable);  //Disable CombFilter
	      op_04=0x00000606; // OPNS[1]=AIO5=1b, OPNS[2]=DAOI(PT3.1)=1b, ADC_CH2 Setting
	      pio_28=0x02020000; // PT3A0=1b, ADC_CH2 Setting
	      DrvADC_SetADCInputChannel(DAO,ADC_Input_AIO1);	//Switch to ADC_CH2 ADC input
	      DrvADC_RefVoltage(AIO2,AIO5); //Set the ADC reference voltage.	//Switch to ADC_CH2 ADC RefVoltage
	      DrvADC_CombFilter(Enable);  //Enable CombFilter
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      ADC_CH1=(ADC_CH1_Array[0]+ ADC_CH1_Array[1]+ ADC_CH1_Array[2]+ ADC_CH1_Array[3])>>2;	//divider by 4
	      CH1_LowByte=(ADC_CH1 >> 7) & 0xFF;
	      CH1_MiddleByte=(ADC_CH1 >> 15) & 0xFF;
	      CH1_HighByte=(ADC_CH1 >> 23) & 0xFF;
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 4:  //ŪkW-1
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH2_Array[0]=ADCData;
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 5:  //ŪkW-2
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH2_Array[1]=ADCData;
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 6:  //ŪkW-3
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH2_Array[2]=ADCData;
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 7:  //ŪkW-4
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH2_Array[3]=ADCData;
	      DrvADC_CombFilter(Disable);  //Disable CombFilter
	      op_04=0x00000600; // OPNS[1]=AIO5=0b, OPNS[2]=DAOI(PT3.1)=0b, cancel ADC_CH2 Setting
	      pio_28=0x02000000; // PT3A0=0b, cancel ADC_CH2 Setting
	      op_04=0x00000A0A; // OPNS[1]=AIO5=1b, OPNS[3]=OPO(PT3.7)=1b, cancel ADC_CH2 Setting
	      pio_24=0x80800000; //PT3IE7=1b, ADC_CH3 Setting
	      op_00=0x00000202; //OPOE=1b, ADC_CH3 Setting
	      DrvADC_SetADCInputChannel(OPO,ADC_Input_AIO1);	//Switch to ADC_CH3 ADC input
	      DrvADC_RefVoltage(AIO2,AIO5); //Set the ADC reference voltage.	//Switch to ADC_CH3 ADC RefVoltage
	      DrvADC_CombFilter(Enable);  //Enable CombFilte
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      ADC_CH2=(ADC_CH2_Array[0]+ ADC_CH2_Array[1]+ ADC_CH2_Array[2]+ ADC_CH2_Array[3])>>2;	//divider by 4
	      CH2_LowByte=(ADC_CH2 >> 7) & 0xFF;
	      CH2_MiddleByte=(ADC_CH2 >> 15) & 0xFF;
	      CH2_HighByte=(ADC_CH2 >> 23) & 0xFF;
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 8:   //ŪkU-1
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH3_Array[0]=ADCData;
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 9:   //ŪkU-2
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH3_Array[1]=ADCData;
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 10:   //ŪkU-3
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH3_Array[2]=ADCData;
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 11:   //ŪkU-4
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH3_Array[3]=ADCData;
	      DrvADC_CombFilter(Disable);  //Disable CombFilter
	      op_04=0x00000A00; // OPNS[1]=AIO5=0b, OPNS[3]=OPO(PT3.7)=0b, cancel ADC_CH3 Setting
	      pio_24=0x80000000; //PT3IE7=0b, cancel ADC_CH3 Setting
	      op_00=0x00000200; //OPOE=0b, cancel ADC_CH3 Setting
	      pmu_00=0x00000200; //ENRFO=0b, ADC_CH4 Setting
	      pio_24=0x40000000; // PT3IE6=0b, ADC_CH4 Setting
	      DrvADC_SetADCInputChannel(ACM,ADC_Input_AIO1);	//Switch to ADC_CH4 ADC input
	      DrvADC_RefVoltage(AIO2,3); //Set the ADC reference voltage.	//Switch to ADC_CH4 ADC RefVoltage
	      DrvADC_CombFilter(Enable);  //Enable CombFilte
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      ADC_CH3=(ADC_CH3_Array[0]+ADC_CH3_Array[1]+ADC_CH3_Array[2]+ADC_CH3_Array[3])>>2;	//divider by 4
	      CH3_LowByte=(ADC_CH3 >> 7) & 0xFF;
	      CH3_MiddleByte=(ADC_CH3 >> 15) & 0xFF;
	      CH3_HighByte=(ADC_CH3 >> 23) & 0xFF;
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 12:  //ŪU-1
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH4_Array[0]=ADCData;
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 13:  //ŪU-2
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH4_Array[1]=ADCData;
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 14:  //ŪU-3
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH4_Array[2]=ADCData;
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      SYS_LowPower(2);  //wait mode
	      break;

	      case 15:  //ŪU-4
	      ADfinish=0;
	      int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	      ADC_CH4_Array[3]=ADCData;
	      DrvADC_CombFilter(Disable);  //Disable CombFilter
	      pmu_00=0x00000202; //ENRFO=1b, Cancel ADC_CH4 Setting
	      pio_24=0x40400000; // PT3IE6=1b, Cancel ADC_CH4 Setting
	      DrvADC_SetADCInputChannel(ADC_Input_AIO3,ADC_Input_AIO1);	//Switch to ADC_CH1 ADC input
	      DrvADC_RefVoltage(AIO2,AIO3); //Set the ADC reference voltage.	//Switch to ADC_CH1 ADC RefVoltage
	      //DrvADC_RefVoltage(AIO2,VSSA); //test only
	      DrvADC_CombFilter(Enable);  //Enable CombFilte
	      DrvADC_ClearIntFlag();
	      DrvADC_EnableInt();
	      ADC_CH4=(ADC_CH4_Array[0]+ADC_CH4_Array[1]+ADC_CH4_Array[2]+ADC_CH4_Array[3])>>2;	//divider by 4
	      CH4_LowByte=(ADC_CH4 >> 7) & 0xFF;
	      CH4_MiddleByte=(ADC_CH4 >> 15) & 0xFF;
	      CH4_HighByte=(ADC_CH4 >> 23) & 0xFF;
	      I2C_Sendbuf[13]=CH1_LowByte;
	      I2C_Sendbuf[14]=CH1_MiddleByte;
	      I2C_Sendbuf[15]=CH1_HighByte;
	      I2C_Sendbuf[16]=CH2_LowByte;
	      I2C_Sendbuf[17]=CH2_MiddleByte;
	      I2C_Sendbuf[18]=CH2_HighByte;
	      I2C_Sendbuf[19]=CH3_LowByte;
	      I2C_Sendbuf[20]=CH3_MiddleByte;
	      I2C_Sendbuf[21]=CH3_HighByte;
	      I2C_Sendbuf[22]=CH4_LowByte;
	      I2C_Sendbuf[23]=CH4_MiddleByte;
	      I2C_Sendbuf[24]=CH4_HighByte;
	      ADC_CH4_Finish=1; //Enable I2C Slave data Buffer Update
	      pio_04=0x00000800; //PT1DO3=0  PT1.3 pull low, check output rate=Interrupt pin
	      pio_04=0x00000808; //PT1DO3=1  PT1.3 pull High, check output rate=Interrupt pin
	      SYS_LowPower(2);  //idle mode
	      break;
	    }
	  }

  while(1) //to do Moving_Average
  {
	  for(i=0;i<=3;i++)
	 	{
	 	  switch (i)
	 	  {

	        case 0:  //ŪW-1
	 	    ADfinish=0;
	 	    int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	 	    ADC_CH1_Array[ADC_CH1_Count]=ADCData;
	 	    DrvADC_CombFilter(Disable);  //Disable CombFilter
	 	  	op_04=0x00000606; // OPNS[1]=AIO5=1b, OPNS[2]=DAOI(PT3.1)=1b, ADC_CH2 Setting
	 	  	pio_28=0x02020000; // PT3A0=1b, ADC_CH2 Setting
	 	  	DrvADC_SetADCInputChannel(DAO,ADC_Input_AIO1);	//Switch to ADC_CH2 ADC input
	 	  	DrvADC_RefVoltage(AIO2,AIO5); //Set the ADC reference voltage.	//Switch to ADC_CH2 ADC RefVoltage
	 	  	DrvADC_CombFilter(Enable);  //Enable CombFilter
	 	  	DrvADC_ClearIntFlag();
	 	  	DrvADC_EnableInt();
	 	    ADC_CH1_Count++;
	 	   	if(ADC_CH1_Count==4)
	 	   	{
	 	   	  ADC_CH1_Count=0;
	 	   	}
	 	    ADC_CH1=(ADC_CH1_Array[0]+ ADC_CH1_Array[1]+ ADC_CH1_Array[2]+ ADC_CH1_Array[3])>>2;	//divider by 4
	 	    CH1_LowByte=(ADC_CH1 >> 7) & 0xFF;
	 	    CH1_MiddleByte=(ADC_CH1 >> 15) & 0xFF;
	 	    CH1_HighByte=(ADC_CH1 >> 23) & 0xFF;
	 	    SYS_LowPower(2);  //wait mode
	 	    break;


	 	    case 1:   //ŪkW-1
	 	    ADfinish=0;
	 	    int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	 	    ADC_CH2_Array[ADC_CH2_Count]=ADCData;
	 	    DrvADC_CombFilter(Disable);  //Disable CombFilter
	 	  	op_04=0x00000600; // OPNS[1]=AIO5=0b, OPNS[2]=DAOI(PT3.1)=0b, cancel ADC_CH2 Setting
	 	  	pio_28=0x02000000; // PT3A0=0b, cancel ADC_CH2 Setting
	 	  	op_04=0x00000A0A; // OPNS[1]=AIO5=1b, OPNS[3]=OPO(PT3.7)=1b, cancel ADC_CH2 Setting
	 	  	pio_24=0x80800000; //PT3IE7=1b, ADC_CH3 Setting
	 	  	op_00=0x00000202; //OPOE=1b, ADC_CH3 Setting
	 	  	DrvADC_SetADCInputChannel(OPO,ADC_Input_AIO1);	//Switch to ADC_CH3 ADC input
	 	  	DrvADC_RefVoltage(AIO2,AIO5); //Set the ADC reference voltage.	//Switch to ADC_CH3 ADC RefVoltage
	 	  	DrvADC_CombFilter(Enable);  //Enable CombFilte
	 	  	DrvADC_ClearIntFlag();
	 	  	DrvADC_EnableInt();
	 	    ADC_CH2_Count++;
	 	   	if(ADC_CH2_Count==4)
	 	   	{
	 	   	  ADC_CH2_Count=0;
	 	   	}
	 	    ADC_CH2=(ADC_CH2_Array[0]+ ADC_CH2_Array[1]+ ADC_CH2_Array[2]+ ADC_CH2_Array[3])>>2;	//divider by 4
	 	    CH2_LowByte=(ADC_CH2 >> 7) & 0xFF;
	 	    CH2_MiddleByte=(ADC_CH2 >> 15) & 0xFF;
	 	    CH2_HighByte=(ADC_CH2 >> 23) & 0xFF;
	 	    SYS_LowPower(2);  //wait mode
	 	    break;

	 	    case 2:   //ŪkU-1
	 	    ADfinish=0;
	 	    int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	 	    ADC_CH3_Array[ADC_CH3_Count]=ADCData;
	 	    DrvADC_CombFilter(Disable);  //Disable CombFilter
	 	  	op_04=0x00000A00; // OPNS[1]=AIO5=0b, OPNS[3]=OPO(PT3.7)=0b, cancel ADC_CH3 Setting
	 	  	pio_24=0x80000000; //PT3IE7=0b, cancel ADC_CH3 Setting
	 	  	op_00=0x00000200; //OPOE=0b, cancel ADC_CH3 Setting
	 	  	pmu_00=0x00000200; //ENRFO=0b, ADC_CH4 Setting
	 	  	pio_24=0x40000000; // PT3IE6=0b, ADC_CH4 Setting
	 	  	DrvADC_SetADCInputChannel(ACM,ADC_Input_AIO1);	//Switch to ADC_CH4 ADC input
	 	  	DrvADC_RefVoltage(AIO2,3); //Set the ADC reference voltage.	//Switch to ADC_CH4 ADC RefVoltage
	 	  	DrvADC_CombFilter(Enable);  //Enable CombFilte
	 	  	DrvADC_ClearIntFlag();
	 	  	DrvADC_EnableInt();
	 	    ADC_CH3_Count++;
	 	   	if(ADC_CH3_Count==4)
	 	   	{
	 	   	  ADC_CH3_Count=0;
	 	   	}
	 	    ADC_CH3=(ADC_CH3_Array[0]+ADC_CH3_Array[1]+ADC_CH3_Array[2]+ADC_CH3_Array[3])>>2;	//divider by 4
	 	    CH3_LowByte=(ADC_CH3 >> 7) & 0xFF;
	 	    CH3_MiddleByte=(ADC_CH3 >> 15) & 0xFF;
	 	    CH3_HighByte=(ADC_CH3 >> 23) & 0xFF;
	 	    SYS_LowPower(2);  //wait mode
	 	    break;


	 	    case 3:   //ŪU-1
	 	    ADfinish=0;
	 	    int_00=0x30003000;  //I2CIE=0, I2CEIE=0
	 	    ADC_CH4_Array[ADC_CH4_Count]=ADCData;
	 	    DrvADC_CombFilter(Disable);  //Disable CombFilter
	 		pmu_00=0x00000202; //ENRFO=1b, Cancel ADC_CH4 Setting
	 		pio_24=0x40400000; // PT3IE6=1b, Cancel ADC_CH4 Setting
	 		DrvADC_SetADCInputChannel(ADC_Input_AIO3,ADC_Input_AIO1);	//Switch to ADC_CH1 ADC input
	 		DrvADC_RefVoltage(AIO2,AIO3); //Set the ADC reference voltage.	//Switch to ADC_CH1 ADC RefVoltage
	 		//DrvADC_RefVoltage(AIO2,VSSA); //test only
	 		DrvADC_CombFilter(Enable);  //Enable CombFilte
	 		DrvADC_ClearIntFlag();
	 		DrvADC_EnableInt();
	 	    ADC_CH4_Count++;
	 	    if(ADC_CH4_Count==4)
	 	    {
	 	      ADC_CH4_Count=0;
	 	    }
	 	    ADC_CH4=(ADC_CH4_Array[0]+ADC_CH4_Array[1]+ADC_CH4_Array[2]+ADC_CH4_Array[3])>>2;	//divider by 4
	 	    CH4_LowByte=(ADC_CH4 >> 7) & 0xFF;
	 	    CH4_MiddleByte=(ADC_CH4 >> 15) & 0xFF;
	 	    CH4_HighByte=(ADC_CH4 >> 23) & 0xFF;
	 	    I2C_Sendbuf[13]=CH1_LowByte;
	 		I2C_Sendbuf[14]=CH1_MiddleByte;
	 		I2C_Sendbuf[15]=CH1_HighByte;
	 		I2C_Sendbuf[16]=CH2_LowByte;
	 		I2C_Sendbuf[17]=CH2_MiddleByte;
	 		I2C_Sendbuf[18]=CH2_HighByte;
	 		I2C_Sendbuf[19]=CH3_LowByte;
	 		I2C_Sendbuf[20]=CH3_MiddleByte;
	 		I2C_Sendbuf[21]=CH3_HighByte;
	 		I2C_Sendbuf[22]=CH4_LowByte;
	 		I2C_Sendbuf[23]=CH4_MiddleByte;
	 		I2C_Sendbuf[24]=CH4_HighByte;
	 	    ADC_CH4_Finish=1; //Enable I2C Slave data Buffer Update
	 	    pio_04=0x00000800; //PT1DO3=0  PT1.3 pull low, check output rate=Interrupt pin
	 	    pio_04=0x00000808; //PT1DO3=1  PT1.3 pull High, check output rate=Interrupt pin
	 	    SYS_LowPower(2);  //idle mode
	 	    break;
	       }
	     }
  }

  asm("NOP");
  return 0;

}

/*---------------------------------------------------------------------------*/
/* Hardware Communication Interrupt                                          */
/* UART/SPI/I2C Interrupt Service Routines                                   */
/*---------------------------------------------------------------------------*/
void HW0_ISR(void)
{
  unsigned char I2C_Status;

  if(DrvI2C_ReadIntFlag()==E_DRVI2C_INT)  // Get I2C Interrupt Flag
  {
    I2C_Status=DrvI2C_GetStatusFlag();  // Get I2C Status Flag
    switch(I2C_Status)
    {
      case 0x44: //SACTFlag+ACKFlag
          { //Own slave A + W has been received. ACK has been transmitted.
            DataRxIndex=0;
            MCUSTATUSbits.b_I2C_RxDone=1;
            DrvI2C_Ctrl(0,0,0,1);  // Set ACK bit
            break;
          }
      case 0x4C: //SACTFlag+DFFlag+ACKFlag
          { //Data byte has been received. ACK has been transmitted.
            I2C_Recbuf[DataRxIndex]=DrvI2C_ReadData();
            if(I2C_Recbuf[DataRxIndex]==0x80)
            {
                if(ADC_CH4_Finish==1)
                {
        	        I2C_Sendbuf[1]=I2C_Sendbuf[13]|0x1; //Ch1 bit0=1b, new data
        	    	I2C_Sendbuf[2]=I2C_Sendbuf[14];
        	    	I2C_Sendbuf[3]=I2C_Sendbuf[15];
        	    	I2C_Sendbuf[4]=I2C_Sendbuf[16]|0x1;  //Ch2 bit0=1b, new data
        	    	I2C_Sendbuf[5]=I2C_Sendbuf[17];
        	    	I2C_Sendbuf[6]=I2C_Sendbuf[18];
        	    	I2C_Sendbuf[7]=I2C_Sendbuf[19]|0x1;  //Ch3 bit0=1b, new data
        	    	I2C_Sendbuf[8]=I2C_Sendbuf[20];
        	    	I2C_Sendbuf[9]=I2C_Sendbuf[21];
        	    	I2C_Sendbuf[10]=I2C_Sendbuf[22]|0x1;  //Ch4 bit0=1b, new data
        	    	I2C_Sendbuf[11]=I2C_Sendbuf[23];
        	    	I2C_Sendbuf[12]=I2C_Sendbuf[24];
                	ADC_CH4_Finish=0;
                }
                else
                {
                	I2C_Sendbuf[1]=I2C_Sendbuf[1]|0x0;  //Ch1 bit0=0b, old data
        	    	I2C_Sendbuf[2]=I2C_Sendbuf[2];
        	    	I2C_Sendbuf[3]=I2C_Sendbuf[3];
        	    	I2C_Sendbuf[4]=I2C_Sendbuf[4]|0x0;  //Ch2 bit0=0b, old data
        	    	I2C_Sendbuf[5]=I2C_Sendbuf[5];
        	    	I2C_Sendbuf[6]=I2C_Sendbuf[6];
        	    	I2C_Sendbuf[7]=I2C_Sendbuf[7]|0x0;  //Ch3 bit0=0b, old data
        	    	I2C_Sendbuf[8]=I2C_Sendbuf[8];
        	    	I2C_Sendbuf[9]=I2C_Sendbuf[9];
        	    	I2C_Sendbuf[10]=I2C_Sendbuf[10]|0x0;  //Ch4 bit0=0b, old data
        	    	I2C_Sendbuf[11]=I2C_Sendbuf[11];
        	    	I2C_Sendbuf[12]=I2C_Sendbuf[12];

                }
            }
            DataRxIndex++;
            DrvI2C_Ctrl(0,0,0,1);  // Set ACK bit
            break;
          }
      case 0x48: //SACTFlag+DFFlag
          { //Data byte has been received. NACK has been transmitted.
            I2C_Recbuf[DataRxIndex++]=DrvI2C_ReadData();
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          }

      case 0x4A: // I2C General Call
          { //One data byte has been received. NACK has been transmitted
        	GeneralFlag=1;
        	DataRxIndex=0;
        	MCUSTATUSbits.b_I2C_RxDone=1;
        	I2C_Recbuf[DataRxIndex++]=DrvI2C_ReadData();
            DrvI2C_Ctrl(0,0,0,1);  // Set ACK bit
            break;
          }
      case 0x4E: // I2C General Call
           { //One data byte has been received. ACK has been transmitted
        	 GeneralFlag=1;
        	 I2C_Recbuf[DataRxIndex++]=DrvI2C_ReadData();
             DrvI2C_Ctrl(0,0,0,1);  // Set ACK bit
             break;
           }

      case 0x54: //SACTFlag+RWFlag+ACKFlag
          { //Own slave A + R has been received. ACK has been transmitted.
            MCUSTATUSbits.b_I2C_TxDone=1;
           	DataTxIndex=0;
            DataTxIndex++;  //Robert Add, Read data index
            DrvI2C_WriteData(I2C_Sendbuf[DataTxIndex++]);  // Send Data to Master
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      case 0x55: //SACTFlag+RWFlag
          { //Own slave A + R has been received. NACK has been transmitted.
            DataTxIndex=0;
            MCUSTATUSbits.b_I2C_TxDone=1;
            DrvI2C_WriteData(I2C_Sendbuf[DataTxIndex++]);  // Send Data to Master
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      case 0x5C:
          { //Data byte has been transmitted. ACK has been transmitted.
            DrvI2C_WriteData(I2C_Sendbuf[DataTxIndex++]);  // Send Data to Master
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      case 0x58:
          { //Data byte has been transmitted. NACK has been received.
            DataTxLen=DataTxIndex;
            EndFlag=1;
            DrvI2C_WriteData(0x80); // set MSB==High, NACK then STOP
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      case 0x30:
          { //A STOP  has been received.
            DataRxLen=DataRxIndex;
            DataTxLen=DataTxIndex;
        	EndFlag=1;
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      default:
          {
            DataTxIndex=0;
            DataRxIndex=0;
            EndFlag=1;
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
    }
    DrvI2C_ClearEIRQ();
    DrvI2C_ClearIntFlag(2); // Clear I2C Interrupt Flag(I2CIF)
  }
  if(DrvI2C_ReadIntFlag()==E_DRVI2C_ERROR_INT)  //Get I2C Error Interrupt Flag
  {
    EndFlag=1;
    DrvI2C_ClearIRQ();
    DrvI2C_ClearIntFlag(2); // Clear I2C Interrupt Flag(I2CEIF)
    DrvI2C_Ctrl(0,1,0,0);   // I2C as master sends STOP signal
  }
}

/*---------------------------------------------------------------------------*/
/* WDT & RTC & Timer A/B/C Interrupt Service Routines                                         */
/*---------------------------------------------------------------------------*/
void HW1_ISR(void)
{


}

/*----------------------------------------------------------------------------*/
/* HW2 ADC Interrupt Subroutines                                              */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{
  if(DrvADC_ReadIntFlag()==1) //ŪADC_nDлx줸
  {
    DrvADC_ClearIntFlag();
	ADCData=DrvADC_GetConversionData()>>8;
	ADfinish=1;
	DrvADC_DisableInt();
    int_00=0x30300000;  //I2CIE=1, I2CEIE=1
  }
}

/*---------------------------------------------------------------------------*/
/* CMP/OPA Interrupt Service Routines                                            */
/*---------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}

/*---------------------------------------------------------------------------*/
/* PT1 Interrupt Service Routines                                            */
/*---------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}

/*---------------------------------------------------------------------------*/
/* PT2 Interrupt Service Routines                                            */
/*---------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}



/*---------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                */
/*---------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}


/*----------------------------------------------------------------------------*/
/* ADC Function                                                               */
/*----------------------------------------------------------------------------*/
void InitalADC0(void)
{
  //Set ADC input pin
  DrvADC_SetADCInputChannel(ADC_Input_AIO3,ADC_Input_AIO1);	//Switch to ADC_CH1 ADC input
  DrvADC_InputSwitch(OPEN);
  DrvADC_RefInputShort(OPEN);
  DrvADC_Gain(0,0);          //Set the ADC Gain 1, PGA=1
  //DrvADC_Gain(1,0);          //Set the ADC Gain 1, PGA=8
  DrvADC_DCoffset(0);        //DC offset input voltage selection
  DrvADC_RefVoltage(AIO2,AIO3); //Set the ADC reference voltage.	//Switch to ADC_CH1 ADC RefVoltage
  //DrvADC_RefVoltage(AIO2,VSSA); //test only
  //DrvADC_FullRefRange(0);    //Set the ADC reference range select.
  DrvADC_FullRefRange(1);    //Set the ADC reference range select.
                             //0:Full reference range input
                             //1:1/2 reference range input
  DrvADC_OSR(8);             //8:OSR=128, Speed=2.56k sps
  DrvADC_ClkEnable(0,1);     //Setting ADC CLOCK ADCK=HS_CK/6
  DrvADC_FastChopper(1);     //fast chopper enable

  //Set VDDA voltage
  DrvPMU_VDDA_LDO_Ctrl(E_LDO);
  DrvPMU_VDDA_Voltage(E_VDDA2_4);
  //DrvPMU_VDDA_Voltage(E_VDDA2_7);
  //DrvPMU_VDDA_Voltage(E_VDDA3_0);
  DrvPMU_BandgapEnable();

  //DrvPMU_REFO_Enable();
  DrvPMU_AnalogGround(Enable);//ADC analog ground source selection.
                              //1:Enable buffer and use internal source
  //DrvPMU_LDO_LowPower(1);     //VDD LDO with low power control.
                              //0 : Normal;1 : Low power
  Delay(0x1000);

  //Set ADC interrupt
  DrvADC_EnableInt();
  DrvADC_ClearIntFlag();
  DrvADC_Enable();
  DrvADC_CombFilter(Enable); //CombFilter open
}

/*--------------------------------------------------------------------*/
/* Hardware I2C Initial                                               */
/*--------------------------------------------------------------------*/
void HWI2C_INI(void)
{

  //DrvI2C_SetIOPin(0);  //Set I2C port, PT1.0=SCL, PT1.1=SDA
  DrvI2C_SetIOPin(7);  //Set I2C port, PT2.6=SCL, PT2.7=SDA
  clk_00=0xff01;  //ENHAO=1

  i2c_00=0xff00ff00;//0x41000//i2c off
  i2c_14=0x00ffffff;//0x44014//Transmitter, Data Buffer=1

  i2c_08=0xff200000; //0x41008, Set I2C CRG
  i2c_00=0xff00ff01;//0x41000//ok i2c enable
  i2c_08=0xff04ff7f; //0x41008//I2C Clock Control//Time-out Control Register

  i2c_0c=0x00ff0020;//I2C slaver id_0=0x20
  i2c_04=0xffff80;//0x41004//i2c slaver en
  i2c_0c=0x00000021;//slaver id_0
  asm("NOP");

  int_00=0xff30ff00;  //I2CIE=1, I2CEIE

}

/*---------------------------------------------------------------------------*/
/* End Of File                                                               */
/*---------------------------------------------------------------------------*/
